@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -77,6 +77,9 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
   /* See if this operator needs any parameters. If so, pop them. */
   switch(op)
     {
+    case GAL_ARITHMETIC_OP_QUANTILE:
+      numparams=1;
+      break;
     case GAL_ARITHMETIC_OP_SIGCLIP_STD:
     case GAL_ARITHMETIC_OP_SIGCLIP_MEAN:
     case GAL_ARITHMETIC_OP_SIGCLIP_MEDIAN:
@@ -93,15 +96,15 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
          to the list. */
       tmp=operands_pop(p, token_string);
       if(tmp->size>1)
-        error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" "
+        error(EXIT_FAILURE, 0, "the %s popped operand of the '%s' "
               "operator must be a single number", cstring, token_string);
       tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_FLOAT32);
       gal_list_data_add(params, tmp);
 
-      /* A small sanity check (none of the parameters for sigma-clipping
-         can be negative.. */
+      /* A small sanity check (none of the parameters for sigma-clipping,
+         or quantile estimation can be negative. */
       if( ((float *)(tmp->array))[0]<=0.0 )
-        error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" "
+        error(EXIT_FAILURE, 0, "the %s popped operand of the '%s' "
               "operator cannot be negative", cstring, token_string);
 
       /* Increment the counter string. */
@@ -111,7 +114,7 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
   /* Check if its a number. */
   numpop=operands_pop(p, token_string);
   if(numpop->size>1)
-    error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" "
+    error(EXIT_FAILURE, 0, "the %s popped operand of the '%s' "
           "operator (number of input datasets) must be a number, not an "
           "array", cstring, token_string);
 
@@ -133,7 +136,7 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
     /* Floating point numbers are not acceptable in this context. */
     case GAL_TYPE_FLOAT32:
     case GAL_TYPE_FLOAT64:
-      error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" "
+      error(EXIT_FAILURE, 0, "the %s popped operand of the '%s' "
             "operator (number of input datasets) must be an integer type",
             cstring, token_string);
 
@@ -144,7 +147,7 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
 
   /* If control reaches here, then the number must have been a negative
      value, so print an error. */
-  error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" operator "
+  error(EXIT_FAILURE, 0, "the %s popped operand of the '%s' operator "
         "cannot be zero or a negative number", cstring,
         token_string);
   return 0;
@@ -219,7 +222,7 @@ arithmetic_filter(void *in_prm)
   /* Go over all the pixels that were assigned to this thread. */
   for(i=0; tprm->indexs[i] != GAL_BLANK_SIZE_T; ++i)
     {
-      /* For easy reading, put the index in `ind'. */
+      /* For easy reading, put the index in 'ind'. */
       ind=tprm->indexs[i];
 
       /* Get the coordinate of the pixel. */
@@ -289,7 +292,7 @@ arithmetic_filter(void *in_prm)
             case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN: sind = 1; break;
             default:
               error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
-                    "%s to fix the problem. The `afp->operator' value "
+                    "%s to fix the problem. The 'afp->operator' value "
                     "%d is not recognized as sigma-clipped median or "
                     "mean", __func__, PACKAGE_BUGREPORT, afp->operator);
             }
@@ -307,7 +310,7 @@ arithmetic_filter(void *in_prm)
 
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
-                "to fix the problem. `afp->operator' code %d is not "
+                "to fix the problem. 'afp->operator' code %d is not "
                 "recognized", PACKAGE_BUGREPORT, __func__,
                 afp->operator);
         }
@@ -494,7 +497,7 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
 
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to fix "
-                "the problem. The `operator' code %d is not recognized",
+                "the problem. The 'operator' code %d is not recognized",
                 PACKAGE_BUGREPORT, __func__, operator);
         }
 
@@ -550,13 +553,13 @@ arithmetic_binary_sanity_checks(gal_data_t *in, gal_data_t *conn,
 {
   int conn_int;
 
-  /* Do proper sanity checks on `conn'. */
+  /* Do proper sanity checks on 'conn'. */
   if(conn->size!=1)
-    error(EXIT_FAILURE, 0, "the first popped operand to `%s' must be a "
+    error(EXIT_FAILURE, 0, "the first popped operand to '%s' must be a "
           "single number. However, it has %zu elements", operator,
           conn->size);
   if(conn->type==GAL_TYPE_FLOAT32 || conn->type==GAL_TYPE_FLOAT64)
-    error(EXIT_FAILURE, 0, "the first popped operand to `%s' is the "
+    error(EXIT_FAILURE, 0, "the first popped operand to '%s' is the "
           "connectivity (a value between 1 and the number of dimensions) "
           "therefore, it must NOT be a floating point", operator);
 
@@ -565,18 +568,18 @@ arithmetic_binary_sanity_checks(gal_data_t *in, gal_data_t *conn,
   conn=gal_data_copy_to_new_type_free(conn, GAL_TYPE_INT32);
   conn_int = *((int32_t *)(conn->array));
   if(conn_int>in->ndim)
-    error(EXIT_FAILURE, 0, "the first popped operand of `%s' (%d) is "
+    error(EXIT_FAILURE, 0, "the first popped operand of '%s' (%d) is "
           "larger than the number of dimensions in the second-popped "
           "operand (%zu)", operator, conn_int, in->ndim);
 
   /* Make sure the array has an unsigned 8-bit type. */
   if(in->type!=GAL_TYPE_UINT8)
-    error(EXIT_FAILURE, 0, "the second popped operand of `%s' doesn't "
+    error(EXIT_FAILURE, 0, "the second popped operand of '%s' doesn't "
           "have an 8-bit unsigned integer type. It must be a binary "
           "dataset (only being equal to zero is checked). You can use "
-          "the `uint8' operator for type conversion", operator);
+          "the 'uint8' operator for type conversion", operator);
 
-  /* Clean up and return the integer value of `conn'. */
+  /* Clean up and return the integer value of 'conn'. */
   gal_data_free(conn);
   return conn_int;
 }
@@ -635,7 +638,7 @@ arithmetic_connected_components(struct arithmeticparams *p, char *token)
   /* Push the result onto the stack. */
   operands_add(p, NULL, out);
 
-  /* Clean up (`conn' was freed in the sanity check). */
+  /* Clean up ('conn' was freed in the sanity check). */
   gal_data_free(in);
 }
 
@@ -684,10 +687,10 @@ arithmetic_invert(struct arithmeticparams *p, char *token)
     case GAL_TYPE_UINT32: do *u32 = UINT32_MAX-*u32; while(++u32<u32f); break;
     case GAL_TYPE_UINT64: do *u64 = UINT64_MAX-*u64; while(++u64<u64f); break;
     default:
-      error(EXIT_FAILURE, 0, "`invert' operand has %s type. `invert' can "
+      error(EXIT_FAILURE, 0, "'invert' operand has %s type. 'invert' can "
             "only take unsigned integer types.\n\nYou can use any of the "
-            "`uint8', `uint16', `uint32', or `uint64' operators to chage "
-            "the type before calling `invert'",
+            "'uint8', 'uint16', 'uint32', or 'uint64' operators to chage "
+            "the type before calling 'invert'",
             gal_type_name(in->type, 1));
     }
 
@@ -700,9 +703,9 @@ arithmetic_invert(struct arithmeticparams *p, char *token)
 
 
 static void
-arithmetic_interpolate(struct arithmeticparams *p, char *token)
+arithmetic_interpolate(struct arithmeticparams *p, int operator, char *token)
 {
-  int num_int;
+  int num_int, interpop;
   gal_data_t *interpolated;
 
   /* First pop the number of nearby neighbors.*/
@@ -711,27 +714,45 @@ arithmetic_interpolate(struct arithmeticparams *p, char *token)
   /* Then pop the actual dataset to interpolate. */
   gal_data_t *in = operands_pop(p, token);
 
-  /* Do proper sanity checks on `num'. */
+  /* Do proper sanity checks on 'num'. */
   if(num->size!=1)
     error(EXIT_FAILURE, 0, "the first popped operand to "
-          "`interpolate-medianngb' must be a single number. However, "
+          "'interpolate-medianngb' must be a single number. However, "
           "it has %zu elements", num->size);
   if(num->type==GAL_TYPE_FLOAT32 || num->type==GAL_TYPE_FLOAT64)
     error(EXIT_FAILURE, 0, "the first popped operand to "
-          "`interpolate-medianngb' is the number of nearby neighbors (a "
+          "'interpolate-medianngb' is the number of nearby neighbors (a "
           "counter, an integer). It must NOT be a floating point.\n\n"
           "If its already an integer, but in a floating point container, "
-          "you can use the `int32' operator to convert it to a 32-bit "
+          "you can use the 'int32' operator to convert it to a 32-bit "
           "integer for example");
 
   /* Convert the given number to a 32-bit integer and read it in. */
   num=gal_data_copy_to_new_type_free(num, GAL_TYPE_INT32);
   num_int = *((int32_t *)(num->array));
 
+  /* Set the interpolation operator. */
+  switch(operator)
+    {
+    case ARITHMETIC_OP_INTERPOLATE_MINNGB:
+      interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MIN;
+      break;
+    case ARITHMETIC_OP_INTERPOLATE_MAXNGB:
+      interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MAX;
+      break;
+    case ARITHMETIC_OP_INTERPOLATE_MEDIANNGB:
+      interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MEDIAN;
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+            "the problem. The operator code %d isn't recognized",
+            __func__, PACKAGE_BUGREPORT, operator);
+    }
+
   /* Call the interpolation function. */
-  interpolated=gal_interpolate_close_neighbors(in, NULL, p->cp.interpmetric,
-                                               num_int, p->cp.numthreads,
-                                               1, 0);
+  interpolated=gal_interpolate_neighbors(in, NULL, p->cp.interpmetric,
+                                         num_int, p->cp.numthreads,
+                                         1, 0, interpop);
 
   /* Clean up and push the interpolated array onto the stack. */
   gal_data_free(in);
@@ -758,28 +779,28 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
 
   /* Small sanity check. */
   if( dimension->ndim!=1 || dimension->size!=1)
-    error(EXIT_FAILURE, 0, "first popped operand of `collapse-*' operators "
+    error(EXIT_FAILURE, 0, "first popped operand of 'collapse-*' operators "
           "(dimension to collapse) must be a single number (single-element, "
           "one-dimensional dataset). But it has %zu dimension(s) and %zu "
           "element(s).", dimension->ndim, dimension->size);
   if(dimension->type==GAL_TYPE_FLOAT32 || dimension->type==GAL_TYPE_FLOAT64)
-    error(EXIT_FAILURE, 0, "first popped operand of `collapse-*' operators "
+    error(EXIT_FAILURE, 0, "first popped operand of 'collapse-*' operators "
           "(dimension to collapse) must have an integer type, but it has "
-          "a floating point type (`%s')", gal_type_name(dimension->type,1));
+          "a floating point type ('%s')", gal_type_name(dimension->type,1));
   dimension=gal_data_copy_to_new_type_free(dimension, GAL_TYPE_LONG);
   dim=((long *)(dimension->array))[0];
   if(dim<0 || dim==0)
-    error(EXIT_FAILURE, 0, "first popped operand of `collapse-*' operators "
+    error(EXIT_FAILURE, 0, "first popped operand of 'collapse-*' operators "
           "(dimension to collapse) must be positive (larger than zero), it "
           "is %ld", dim);
   if(dim > input->ndim)
-    error(EXIT_FAILURE, 0, "input dataset to `%s' has %zu dimension(s), "
+    error(EXIT_FAILURE, 0, "input dataset to '%s' has %zu dimension(s), "
           "but you have asked to collapse along dimension %zu", token,
           input->ndim, dim);
 
 
   /* If a WCS structure has been read, we'll need to pass it to
-     `gal_dimension_collapse', so it modifies it respectively. */
+     'gal_dimension_collapse', so it modifies it respectively. */
   if(p->wcs_collapsed==0)
     {
       p->wcs_collapsed=1;
@@ -818,9 +839,9 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
 
 
   /* If a WCS structure existed, a modified WCS is now present in
-     `collapsed->wcs'. So we'll let the freeing of `input' free the old
-     `p->refdata.wcs' structure and we'll put the new one there, then we'll
-     set `collapsed->wcs' to `NULL', so the new one isn't freed. */
+     'collapsed->wcs'. So we'll let the freeing of 'input' free the old
+     'p->refdata.wcs' structure and we'll put the new one there, then we'll
+     set 'collapsed->wcs' to 'NULL', so the new one isn't freed. */
   if(collapsed->wcs)
     {
       p->refdata.wcs = collapsed->wcs;
@@ -829,9 +850,9 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
 
 
   /* We'll also need to correct the size of the reference dataset if it
-     hasn't been corrected yet. We'll use `memcpy' to write the new `dsize'
+     hasn't been corrected yet. We'll use 'memcpy' to write the new 'dsize'
      values into the old ones. The dimensions have decreased, so we won't
-     be writing outside of allocated space that `p->refdata.dsize' points
+     be writing outside of allocated space that 'p->refdata.dsize' points
      to. */
   if( p->refdata.ndim != collapsed->ndim )
     {
@@ -903,10 +924,10 @@ arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
 
   /* Make sure the first operand is a number. */
   if(tmp->size!=1)
-    error(EXIT_FAILURE, 0, "first popped operand to `%s' must be a "
+    error(EXIT_FAILURE, 0, "first popped operand to '%s' must be a "
           "number (specifying how many datasets to use)", token);
 
-  /* Put the value into `num'. */
+  /* Put the value into 'num'. */
   tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_SIZE_T);
   num=*(size_t *)(tmp->array);
   gal_data_free(tmp);
@@ -922,14 +943,14 @@ arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
         {
           /* Basic sanity checks. */
           if(tmp->type!=out->type)
-            error(EXIT_FAILURE, 0, "the operands to `%s' have to have the "
+            error(EXIT_FAILURE, 0, "the operands to '%s' have to have the "
                   "same data type (the inputs contain atleast two types: "
-                  "`%s' and `%s')", token, gal_type_name(tmp->type, 1),
+                  "'%s' and '%s')", token, gal_type_name(tmp->type, 1),
                   gal_type_name(out->type, 1));
           if( tmp->ndim!=out->ndim-1
               || tmp->dsize[0]!=out->dsize[1]
               || tmp->dsize[1]!=out->dsize[2] )
-            error(EXIT_FAILURE, 0, "the operands to `%s' have to have the "
+            error(EXIT_FAILURE, 0, "the operands to '%s' have to have the "
                   "same size", token);
         }
       else  /* First popped operand. */
@@ -937,7 +958,7 @@ arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
           /* First popped operand, do necessary basic checks here. */
           if(tmp->ndim!=2)
             error(EXIT_FAILURE, 0, "currently only 2-dimensional datasets "
-                  "are acceptable for `%s', please get in touch with us at "
+                  "are acceptable for '%s', please get in touch with us at "
                   "%s so we add functionality for different dimensions",
                   token, PACKAGE_BUGREPORT);
 
@@ -1016,6 +1037,10 @@ arithmetic_set_operator(char *string, size_t *num_operands)
         { op=ARITHMETIC_OP_FILL_HOLES;            *num_operands=0; }
       else if (!strcmp(string, "invert"))
         { op=ARITHMETIC_OP_INVERT;                *num_operands=0; }
+      else if (!strcmp(string, "interpolate-minngb"))
+        { op=ARITHMETIC_OP_INTERPOLATE_MINNGB;    *num_operands=0; }
+      else if (!strcmp(string, "interpolate-maxngb"))
+        { op=ARITHMETIC_OP_INTERPOLATE_MAXNGB;    *num_operands=0; }
       else if (!strcmp(string, "interpolate-medianngb"))
         { op=ARITHMETIC_OP_INTERPOLATE_MEDIANNGB; *num_operands=0; }
       else if (!strcmp(string, "collapse-sum"))
@@ -1033,7 +1058,7 @@ arithmetic_set_operator(char *string, size_t *num_operands)
       else if (!strcmp(string, "add-dimension"))
         { op=ARITHMETIC_OP_ADD_DIMENSION;         *num_operands=0; }
       else
-        error(EXIT_FAILURE, 0, "the argument \"%s\" could not be "
+        error(EXIT_FAILURE, 0, "the argument '%s' could not be "
               "interpretted as a file name, named dataset, number, "
               "or operator", string);
     }
@@ -1056,7 +1081,7 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
   int flags = ( GAL_ARITHMETIC_INPLACE | GAL_ARITHMETIC_FREE
                 | GAL_ARITHMETIC_NUMOK );
 
-  /* When `num_operands!=0', the operator is in the library. */
+  /* When 'num_operands!=0', the operator is in the library. */
   if(num_operands)
     {
       /* Pop the necessary number of operators. Note that the
@@ -1086,7 +1111,7 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
              operand. So except for sigma-clipping (that has other
              parameters), the first popped operand must be an
              integer number, we will use that to construct a linked
-             list of any number of operands within the single `d1'
+             list of any number of operands within the single 'd1'
              pointer. */
           numop=pop_number_of_operands(p, operator, operator_string, &d2);
           for(i=0;i<numop;++i)
@@ -1095,12 +1120,12 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
 
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-                "the problem. `%zu' is not recognized as an operand "
-                "counter (with `%s')", __func__, PACKAGE_BUGREPORT,
+                "the problem. '%zu' is not recognized as an operand "
+                "counter (with '%s')", __func__, PACKAGE_BUGREPORT,
                 num_operands, operator_string);
         }
 
-      /* Run the arithmetic operation. Note that `gal_arithmetic'
+      /* Run the arithmetic operation. Note that 'gal_arithmetic'
          is a variable argument function (like printf). So the
          number of arguments it uses depend on the operator. So
          when the operator doesn't need three operands, the extra
@@ -1139,8 +1164,10 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           arithmetic_invert(p, operator_string);
           break;
 
+        case ARITHMETIC_OP_INTERPOLATE_MINNGB:
+        case ARITHMETIC_OP_INTERPOLATE_MAXNGB:
         case ARITHMETIC_OP_INTERPOLATE_MEDIANNGB:
-          arithmetic_interpolate(p, operator_string);
+          arithmetic_interpolate(p, operator, operator_string);
           break;
 
         case ARITHMETIC_OP_COLLAPSE_SUM:
@@ -1162,7 +1189,7 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! please contact us at "
                 "%s to fix the problem. The code %d is not "
-                "recognized for `op'", __func__, PACKAGE_BUGREPORT,
+                "recognized for 'op'", __func__, PACKAGE_BUGREPORT,
                 operator);
         }
     }
@@ -1195,7 +1222,7 @@ reversepolish(struct arithmeticparams *p)
   /* Go over each input token and do the work. */
   for(token=p->tokens;token!=NULL;token=token->next)
     {
-      /* The `tofile-' operator's string can end in a `.fits', similar to a
+      /* The 'tofile-' operator's string can end in a '.fits', similar to a
          FITS file input file. So, it needs to be checked before checking
          for a filename. If we have a name or number, then add it to the
          operands linked list. Otherwise, pull out two members and do the
@@ -1240,10 +1267,10 @@ reversepolish(struct arithmeticparams *p)
     error(EXIT_FAILURE, 0, "too many operands");
 
 
-  /* If the final operand has a filename, but its `data' element is NULL,
+  /* If the final operand has a filename, but its 'data' element is NULL,
      then the file hasn't actually be read yet. In this case, we need to
      read the contents of the file and put the resulting dataset into the
-     operands `data' element. This can happen for example if no operators
+     operands 'data' element. This can happen for example if no operators
      are called and there is only one filename as an argument (which can
      happen in scripts). */
   if(p->operands->data==NULL && p->operands->filename)
@@ -1263,8 +1290,8 @@ reversepolish(struct arithmeticparams *p)
         }
       else
         error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to fix "
-              "the problem. While `operands->data' is NULL, the filename "
-              "(`%s') is not recognized as a FITS file", __func__,
+              "the problem. While 'operands->data' is NULL, the filename "
+              "('%s') is not recognized as a FITS file", __func__,
               PACKAGE_BUGREPORT, filename);
     }
 
@@ -1284,7 +1311,7 @@ reversepolish(struct arithmeticparams *p)
   else
     {
       /* Put a copy of the WCS structure from the reference image, it
-         will be freed while freeing `data'. */
+         will be freed while freeing 'data'. */
       data->wcs=p->refdata.wcs;
       if(data->ndim==1 && p->onedasimage==0)
         gal_table_write(data, NULL, p->cp.tableformat,
@@ -1297,8 +1324,8 @@ reversepolish(struct arithmeticparams *p)
     }
 
 
-  /* Clean up, note that above, we copied the pointer to `refdata->wcs'
-     into `data', so it is freed when freeing `data'. */
+  /* Clean up, note that above, we copied the pointer to 'refdata->wcs'
+     into 'data', so it is freed when freeing 'data'. */
   gal_data_free(data);
   free(p->refdata.dsize);
   gal_list_data_free(p->named);
