@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2019, Free Software Foundation, Inc.
+Copyright (C) 2018-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -169,7 +169,7 @@ parse_spectrum_pepare(struct mkcatalog_passparams *pp, size_t *start_end_inc,
                          p->values->unit, "Error in SUM_OTHER column.");
 
   /* Fill up the contents of the first element (note that the first
-     `gal_data_t' is actually in an array, so the skeleton is already
+     'gal_data_t' is actually in an array, so the skeleton is already
      allocated, we just have to allocate its contents. */
   gal_data_initialize(pp->spectrum, NULL, p->specsliceinfo->type, 1,
                       &numslices, NULL, 0, p->cp.minmapsize,
@@ -252,8 +252,8 @@ parse_spectrum_end(struct mkcatalog_passparams *pp, gal_data_t *xybin)
       osearr[i] = sqrt( osearr[i] );
     }
 
-  /* Convert the `double' type columns to `float'. The extra precision of
-     `double' was necessary when we were summing values in each slice. But
+  /* Convert the 'double' type columns to 'float'. The extra precision of
+     'double' was necessary when we were summing values in each slice. But
      afterwards, it is not necessary at all (the measurement error is much
      larger than a double-precision floating point number (15
      decimals). But the extra space gained (double) is very useful in not
@@ -271,7 +271,7 @@ parse_spectrum_end(struct mkcatalog_passparams *pp, gal_data_t *xybin)
   parse_spectrum_uint32_to_best_type(&oarea);
 
   /* List the datasets and write them into the pointer for this object
-     (exact copy of the statement in `parse_spectrum_pepare'). */
+     (exact copy of the statement in 'parse_spectrum_pepare'). */
   pp->spectrum->next->next                       = area;
   area->next                                     = sum;
   area->next->next                               = esum;
@@ -325,14 +325,14 @@ parse_spectrum(struct mkcatalog_passparams *pp, gal_data_t *xybin)
   osarr  = area->next->next->next->next->next->next->array;
   osearr = area->next->next->next->next->next->next->next->array;
 
-  /* If tile-id isn't necessary, set `tid' to a blank value. */
+  /* If tile-id isn't necessary, set 'tid' to a blank value. */
   tid = (p->std && p->std->size>1 && st_std == NULL) ? 0 : GAL_BLANK_SIZE_T;
 
   /* Parse each contiguous patch of memory covered by this object. */
   while( start_end_inc[0] + increment <= start_end_inc[1] )
     {
       /* Set the contiguous range to parse. The pixel-to-pixel counting
-         along the fastest dimension will be done over the `O' pointer. */
+         along the fastest dimension will be done over the 'O' pointer. */
       if( p->values        ) V  = st_v   + increment;
       if( p->std && st_std ) ST = st_std + increment;
       OO = ( O = st_o + increment ) + pp->tile->dsize[ndim-1];
@@ -357,7 +357,7 @@ parse_spectrum(struct mkcatalog_passparams *pp, gal_data_t *xybin)
 
                   /* Get the error associated with this voxel. Note that if
                      we are given a variance dataset already, there is no
-                     need to use `st*st', we can directly use `sval'. */
+                     need to use 'st*st', we can directly use 'sval'. */
                   sval = st_std ? *ST : (p->std->size>1?std[tid]:std[0]);
                   st = p->variance ? sqrt(sval) : sval;
                   var = (p->variance ? sval : st*st) + fabs(*V);
@@ -365,8 +365,8 @@ parse_spectrum(struct mkcatalog_passparams *pp, gal_data_t *xybin)
               else var = NAN;
 
 
-              /* Projected spectra: see if we have a value of `2' in the
-                 `xybin' array (showing that there is atleast one non-blank
+              /* Projected spectra: see if we have a value of '2' in the
+                 'xybin' array (showing that there is atleast one non-blank
                  element there over the whole spectrum.  */
               ++nproj;
               parr [ sind ] += *V;
@@ -402,8 +402,8 @@ parse_spectrum(struct mkcatalog_passparams *pp, gal_data_t *xybin)
       increment += ( gal_tile_block_increment(p->objects, tsize,
                                               num_increment++, NULL) );
 
-      /* Increment the slice number, `sind', and reset the projection (2D)
-         index `pind' if we have just finished parsing a slice. */
+      /* Increment the slice number, 'sind', and reset the projection (2D)
+         index 'pind' if we have just finished parsing a slice. */
       if( (num_increment-1)%pp->tile->dsize[1]==0 )
         {
           /* If there was no measurement, set NaN for the values and their
@@ -447,7 +447,7 @@ parse_objects(struct mkcatalog_passparams *pp)
   float var, sval, varval, skyval, *V=NULL, *SK=NULL, *ST=NULL;
   float *std=p->std?p->std->array:NULL, *sky=p->sky?p->sky->array:NULL;
 
-  /* If tile processing isn't necessary, set `tid' to a blank value. */
+  /* If tile processing isn't necessary, set 'tid' to a blank value. */
   size_t tid = ( ( (p->sky     && p->sky->size>1 && pp->st_sky == NULL )
                    || ( p->std && p->std->size>1 && pp->st_std == NULL ) )
                  ? 0 : GAL_BLANK_SIZE_T );
@@ -496,7 +496,7 @@ parse_objects(struct mkcatalog_passparams *pp)
   while( pp->start_end_inc[0] + increment <= pp->start_end_inc[1] )
     {
       /* Set the contiguous range to parse. The pixel-to-pixel counting
-         along the fastest dimension will be done over the `O' pointer. */
+         along the fastest dimension will be done over the 'O' pointer. */
       if( p->clumps            ) C  = pp->st_c   + increment;
       if( p->values            ) V  = pp->st_v   + increment;
       if( p->sky && pp->st_sky ) SK = pp->st_sky + increment;
@@ -639,9 +639,9 @@ parse_objects(struct mkcatalog_passparams *pp)
                     }
                   /* For each pixel, we have a sky contribution to the
                      counts and the signal's contribution. The standard
-                     deviation in the sky is simply `sval', but the
+                     deviation in the sky is simply 'sval', but the
                      standard deviation of the signal (independent of the
-                     sky) is `sqrt(*V)'. Therefore the total variance of
+                     sky) is 'sqrt(*V)'. Therefore the total variance of
                      this pixel is the variance of the sky added with the
                      absolute value of its sky-subtracted flux. We use the
                      absolute value, because especially as the signal gets
@@ -669,7 +669,7 @@ parse_objects(struct mkcatalog_passparams *pp)
                                               num_increment++, NULL) );
 
       /* If a 2D projection is requested, see if we should initialize (set
-         to zero) the projection-index (`pind') not. */
+         to zero) the projection-index ('pind') not. */
       if(xybin && (num_increment-1)%tsize[1]==0 )
         pind=0;
     }
@@ -741,7 +741,7 @@ parse_clumps(struct mkcatalog_passparams *pp)
   int32_t *objects=p->objects->array, *clumps=p->clumps->array;
   float *std=p->std?p->std->array:NULL, *sky=p->sky?p->sky->array:NULL;
 
-  /* If tile processing isn't necessary, set `tid' to a blank value. */
+  /* If tile processing isn't necessary, set 'tid' to a blank value. */
   size_t tid = ( ( (p->sky     && p->sky->size>1 && pp->st_sky == NULL )
                    || ( p->std && p->std->size>1 && pp->st_std == NULL ) )
                  ? 0 : GAL_BLANK_SIZE_T );
@@ -797,7 +797,7 @@ parse_clumps(struct mkcatalog_passparams *pp)
   while( pp->start_end_inc[0] + increment <= pp->start_end_inc[1] )
     {
       /* Set the contiguous range to parse. The pixel-to-pixel counting
-         along the fastest dimension will be done over the `O' pointer. */
+         along the fastest dimension will be done over the 'O' pointer. */
       C = pp->st_c + increment;
       if( p->values            ) V  = pp->st_v   + increment;
       if( p->sky && pp->st_sky ) SK = pp->st_sky + increment;
@@ -869,7 +869,7 @@ parse_clumps(struct mkcatalog_passparams *pp)
                         }
                     }
 
-                  /* Value related measurements, see `parse_objects' for
+                  /* Value related measurements, see 'parse_objects' for
                      comments. */
                   goodvalue=0;
                   if( p->values && !( p->hasblank && isnan(*V) ) )
@@ -917,7 +917,7 @@ parse_clumps(struct mkcatalog_passparams *pp)
                     }
 
                   /* Sky Standard deviation based measurements, see
-                     `parse_objects' for comments. */
+                     'parse_objects' for comments. */
                   if(p->std)
                     {
                       sval = ( pp->st_std
@@ -947,10 +947,10 @@ parse_clumps(struct mkcatalog_passparams *pp)
                   /* We are on a diffuse (possibly a river) pixel. So the
                      value of this pixel has to be added to any of the
                      clumps in touches. But since it might touch a labeled
-                     region more than once, we use `ngblabs' to keep track
+                     region more than once, we use 'ngblabs' to keep track
                      of which label we have already added its value
-                     to. `ii' is the number of different labels this river
-                     pixel has already been considered for. `ngblabs' will
+                     to. 'ii' is the number of different labels this river
+                     pixel has already been considered for. 'ngblabs' will
                      keep the list labels. */
                   ii=0;
                   memset(ngblabs, 0, nngb*sizeof *ngblabs);
@@ -1019,7 +1019,7 @@ parse_clumps(struct mkcatalog_passparams *pp)
                                               num_increment++, NULL) );
 
       /* If a 2D projection is requested, see if we should initialize (set
-         to zero) the projection-index (`pind') not. */
+         to zero) the projection-index ('pind') not. */
       if(xybin && (num_increment-1) % tsize[1]==0 )
         pind=0;
     }
@@ -1062,29 +1062,56 @@ parse_clumps(struct mkcatalog_passparams *pp)
 
 
 void
-parse_median(struct mkcatalog_passparams *pp)
+parse_order_based(struct mkcatalog_passparams *pp)
 {
   struct mkcatalogparams *p=pp->p;
 
   float *V;
   double *ci;
-  gal_data_t *median;
+  float *sigcliparr;
+  gal_data_t *result;
   int32_t *O, *OO, *C=NULL;
-  gal_data_t **clumpsmed=NULL;
   size_t i, increment=0, num_increment=1;
+  gal_data_t *objvals=NULL, **clumpsvals=NULL;
   size_t *tsize=pp->tile->dsize, ndim=p->objects->ndim;
   size_t counter=0, *ccounter=NULL, tmpsize=pp->oi[OCOL_NUM];
-  gal_data_t *objmed=gal_data_alloc(NULL, p->values->type, 1, &tmpsize, NULL,
-                                    0, p->cp.minmapsize, p->cp.quietmmap,
-                                    NULL, NULL, NULL);
 
-  /* Allocate space for the clump medians. */
+  /* It may happen that there are no usable pixels for this object (and
+     thus its possible clumps). In this case `tmpsize' will be zero and we
+     can just write NaN values for the necessary columns. */
+  if(tmpsize==0)
+    {
+      if(p->oiflag[ OCOL_MEDIAN        ]) pp->oi[ OCOL_MEDIAN       ] = NAN;
+      if(p->oiflag[ OCOL_SIGCLIPNUM    ]) pp->oi[ OCOL_SIGCLIPNUM   ] = 0;
+      if(p->oiflag[ OCOL_SIGCLIPSTD    ]) pp->oi[ OCOL_SIGCLIPSTD   ] = 0;
+      if(p->oiflag[ OCOL_SIGCLIPMEAN   ]) pp->oi[ OCOL_SIGCLIPMEAN  ] = NAN;
+      if(p->oiflag[ OCOL_SIGCLIPMEDIAN ]) pp->oi[ OCOL_SIGCLIPMEDIAN] = NAN;
+      if(p->clumps)
+        for(i=0;i<pp->clumpsinobj;++i)
+          {
+            ci=&pp->ci[ i * CCOL_NUMCOLS ];
+            if(p->ciflag[ CCOL_MEDIAN        ]) ci[ CCOL_MEDIAN      ] = NAN;
+            if(p->ciflag[ CCOL_SIGCLIPNUM    ]) ci[ CCOL_SIGCLIPNUM  ] = 0;
+            if(p->ciflag[ CCOL_SIGCLIPSTD    ]) ci[ CCOL_SIGCLIPSTD  ] = 0;
+            if(p->ciflag[ CCOL_SIGCLIPMEAN   ]) ci[ CCOL_SIGCLIPMEAN ] = NAN;
+            if(p->ciflag[ CCOL_SIGCLIPMEDIAN ]) ci[CCOL_SIGCLIPMEDIAN] = NAN;
+          }
+      return;
+    }
+
+  /* We know we have pixels to use, so allocate space for the values within
+     the object. */
+  objvals=gal_data_alloc(NULL, p->values->type, 1, &tmpsize, NULL, 0,
+                         p->cp.minmapsize, p->cp.quietmmap, NULL, NULL,
+                         NULL);
+
+  /* Allocate space for the clump values. */
   if(p->clumps)
     {
       errno=0;
-      clumpsmed=malloc(pp->clumpsinobj * sizeof *clumpsmed);
-      if(clumpsmed==NULL)
-        error(EXIT_FAILURE, errno, "%s: couldn't allocate `clumpsmed' for "
+      clumpsvals=malloc(pp->clumpsinobj * sizeof *clumpsvals);
+      if(clumpsvals==NULL)
+        error(EXIT_FAILURE, errno, "%s: couldn't allocate 'clumpsvals' for "
               "%zu clumps", __func__, pp->clumpsinobj);
 
 
@@ -1094,9 +1121,9 @@ parse_median(struct mkcatalog_passparams *pp)
       for(i=0;i<pp->clumpsinobj;++i)
         {
           tmpsize=pp->ci[ i * CCOL_NUMCOLS + CCOL_NUM ];
-          clumpsmed[i]=gal_data_alloc(NULL, p->values->type, 1, &tmpsize,
-                                      NULL, 0, p->cp.minmapsize,
-                                      p->cp.quietmmap, NULL, NULL, NULL);
+          clumpsvals[i]=gal_data_alloc(NULL, p->values->type, 1, &tmpsize,
+                                       NULL, 0, p->cp.minmapsize,
+                                       p->cp.quietmmap, NULL, NULL, NULL);
         }
     }
 
@@ -1105,7 +1132,7 @@ parse_median(struct mkcatalog_passparams *pp)
   while( pp->start_end_inc[0] + increment <= pp->start_end_inc[1] )
     {
       /* Set the contiguous range to parse. The pixel-to-pixel counting
-         along the fastest dimension will be done over the `O' pointer. */
+         along the fastest dimension will be done over the 'O' pointer. */
       V = pp->st_v + increment;
       if(p->clumps) C = pp->st_c + increment;
       OO = ( O = pp->st_o + increment ) + tsize[ndim-1];
@@ -1114,20 +1141,20 @@ parse_median(struct mkcatalog_passparams *pp)
       do
         {
           /* If this pixel belongs to the requested object, then do the
-             processing. `hasblank' is constant, so when the values doesn't
-             have any blank values, the `isnan' will never be checked. */
+             processing. 'hasblank' is constant, so when the values doesn't
+             have any blank values, the 'isnan' will never be checked. */
           if( *O==pp->object && !( p->hasblank && isnan(*V) ) )
             {
               /* Copy the value for the whole object. */
-              memcpy( gal_pointer_increment(objmed->array, counter++,
+              memcpy( gal_pointer_increment(objvals->array, counter++,
                                              p->values->type), V,
                       gal_type_sizeof(p->values->type) );
 
               /* We are also on a clump. */
               if(p->clumps && *C>0)
-                memcpy( gal_pointer_increment(clumpsmed[*C-1]->array,
-                                               ccounter[*C-1]++,
-                                               p->values->type), V,
+                memcpy( gal_pointer_increment(clumpsvals[*C-1]->array,
+                                              ccounter[*C-1]++,
+                                              p->values->type), V,
                         gal_type_sizeof(p->values->type) );
             }
 
@@ -1143,28 +1170,89 @@ parse_median(struct mkcatalog_passparams *pp)
     }
 
 
-  /* Calculate the final medians for objects. */
-  median=gal_data_copy_to_new_type_free(gal_statistics_median(objmed, 1),
-                                        GAL_TYPE_FLOAT64);
-  pp->oi[OCOL_MEDIAN]=*((double *)(median->array));
-  gal_data_free(objmed);
-  gal_data_free(median);
+  /* Calculate the necessary values for the objects. */
+  if(p->oiflag[ OCOL_MEDIAN ])
+    {
+      result=gal_data_copy_to_new_type_free(gal_statistics_median(objvals, 1),
+                                            GAL_TYPE_FLOAT64);
+      pp->oi[OCOL_MEDIAN]=*((double *)(result->array));
+      gal_data_free(result);
+    }
+  if(p->oiflag[ OCOL_SIGCLIPNUM ]
+     || p->oiflag[ OCOL_SIGCLIPSTD ]
+     || p->oiflag[ OCOL_SIGCLIPMEAN ]
+     || p->oiflag[ OCOL_SIGCLIPMEDIAN ])
+    {
+      /* Calculate the sigma-clipped results and write them in any
+         requested column. */
+      result=gal_statistics_sigma_clip(objvals, p->sigmaclip[0],
+                                       p->sigmaclip[1], 1, 1);
+      sigcliparr=result->array;
+      if(p->oiflag[ OCOL_SIGCLIPNUM ])
+        pp->oi[OCOL_SIGCLIPNUM]=sigcliparr[0];
+      if(p->oiflag[ OCOL_SIGCLIPSTD ])
+        pp->oi[OCOL_SIGCLIPSTD]=sigcliparr[3];
+      if(p->oiflag[ OCOL_SIGCLIPMEAN ])
+        pp->oi[OCOL_SIGCLIPMEAN]=sigcliparr[2];
+      if(p->oiflag[ OCOL_SIGCLIPMEDIAN ])
+        pp->oi[OCOL_SIGCLIPMEDIAN]=sigcliparr[1];
+
+      /* Clean up the sigma-clipped values. */
+      gal_data_free(result);
+    }
 
 
-  /* Calculate the median for clumps. */
+  /* Clean up the object values. */
+  gal_data_free(objvals);
+
+
+  /* Calculate the necessary value for clumps. */
   if(p->clumps)
     {
       for(i=0;i<pp->clumpsinobj;++i)
         {
+          /* Set the main row to fill. */
           ci=&pp->ci[ i * CCOL_NUMCOLS ];
-          median=gal_statistics_median(clumpsmed[i], 1);
-          median=gal_data_copy_to_new_type_free(median, GAL_TYPE_FLOAT64);
-          ci[ CCOL_MEDIAN ] = ( *((double *)(median->array))
-                                - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]) );
-          gal_data_free(clumpsmed[i]);
-          gal_data_free(median);
+
+          /* Do the necessary calculation. */
+          if(p->ciflag[ CCOL_MEDIAN ])
+            {
+              result=gal_statistics_median(clumpsvals[i], 1);
+              result=gal_data_copy_to_new_type_free(result, GAL_TYPE_FLOAT64);
+              ci[ CCOL_MEDIAN ] = ( *((double *)(result->array))
+                                    - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]) );
+              gal_data_free(result);
+            }
+          if(p->ciflag[ CCOL_SIGCLIPNUM ]
+             || p->ciflag[ CCOL_SIGCLIPSTD ]
+             || p->ciflag[ CCOL_SIGCLIPMEAN ]
+             || p->ciflag[ CCOL_SIGCLIPMEDIAN ])
+            {
+              /* Calculate the sigma-clipped results and write them in any
+                 requested column. */
+              result=gal_statistics_sigma_clip(clumpsvals[i], p->sigmaclip[0],
+                                               p->sigmaclip[1], 1, 1);
+              sigcliparr=result->array;
+              if(p->ciflag[ CCOL_SIGCLIPNUM ])
+                ci[CCOL_SIGCLIPNUM]=sigcliparr[0];
+              if(p->ciflag[ CCOL_SIGCLIPSTD ])
+                ci[CCOL_SIGCLIPSTD]=( sigcliparr[3]
+                                      - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
+              if(p->ciflag[ CCOL_SIGCLIPMEAN ])
+                ci[CCOL_SIGCLIPMEAN]=( sigcliparr[2]
+                                       - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
+              if(p->ciflag[ CCOL_SIGCLIPMEDIAN ])
+                ci[CCOL_SIGCLIPMEDIAN]=( sigcliparr[1]
+                                         - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
+
+              /* Clean up the sigma-clipped values. */
+              gal_data_free(result);
+            }
+
+          /* Clean up this clump's values. */
+          gal_data_free(clumpsvals[i]);
         }
-      free(clumpsmed);
+      free(clumpsvals);
       free(ccounter);
     }
 }
