@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2019, Free Software Foundation, Inc.
+Copyright (C) 2016-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,11 +31,13 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gsl/gsl_heapsort.h>
 
+#include <gnuastro/txt.h>
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/table.h>
 #include <gnuastro/qsort.h>
 #include <gnuastro/pointer.h>
+#include <gnuastro/polygon.h>
 #include <gnuastro/arithmetic.h>
 #include <gnuastro/statistics.h>
 #include <gnuastro/permutation.h>
@@ -87,7 +89,7 @@ table_selection_range(struct tableparams *p, gal_data_t *col)
   /* First, make sure everything is OK. */
   if(p->range==NULL)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us to fix the "
-          "problem at %s. `p->range' should not be NULL at this point",
+          "problem at %s. 'p->range' should not be NULL at this point",
           __func__, PACKAGE_BUGREPORT);
 
   /* Allocations. */
@@ -101,7 +103,7 @@ table_selection_range(struct tableparams *p, gal_data_t *col)
   ((double *)(min->array))[0] = darr[0];
   ((double *)(max->array))[0] = darr[1];
 
-  /* Move `p->range' to the next element in the list and free the current
+  /* Move 'p->range' to the next element in the list and free the current
      one (we have already read its values and don't need it any more). */
   tmp=p->range;
   p->range=p->range->next;
@@ -135,11 +137,121 @@ table_selection_range(struct tableparams *p, gal_data_t *col)
 
 
 
+/* Read column value of any type as a double for the polygon options. */
+static double
+selection_polygon_read_point(gal_data_t *col, size_t i)
+{
+  /* Check and assign the points to the points array. */
+  switch(col->type)
+    {
+    case GAL_TYPE_INT8:    return (( int8_t   *)col->array)[i];
+    case GAL_TYPE_UINT8:   return (( uint8_t  *)col->array)[i];
+    case GAL_TYPE_UINT16:  return (( uint16_t *)col->array)[i];
+    case GAL_TYPE_INT16:   return (( int16_t  *)col->array)[i];
+    case GAL_TYPE_UINT32:  return (( uint32_t *)col->array)[i];
+    case GAL_TYPE_INT32:   return (( int32_t  *)col->array)[i];
+    case GAL_TYPE_UINT64:  return (( uint64_t *)col->array)[i];
+    case GAL_TYPE_INT64:   return (( int64_t  *)col->array)[i];
+    case GAL_TYPE_FLOAT32: return (( float    *)col->array)[i];
+    case GAL_TYPE_FLOAT64: return (( double   *)col->array)[i];
+    default:
+      error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
+            __func__, col->type);
+    }
+
+  /* Control should not reach here. */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
+        "problem. Control should not reach the end of this function",
+        __func__, PACKAGE_BUGREPORT);
+  return NAN;
+}
+
+
+
+
+
+/* Mask the rows that are not in the given polygon. */
+static gal_data_t *
+table_selection_polygon(struct tableparams *p, gal_data_t *col1,
+                        gal_data_t *col2, int in1out0)
+{
+  uint8_t *oarr;
+  double point[2];
+  gal_data_t *out=NULL;
+  size_t i, psize=p->polygon->size/2;
+
+  /* Allocate the output array: This array will have a '0' for the points
+     which are inside the polygon and '1' for those that are outside of it
+     (to be masked/removed from the input). */
+  out=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, col1->dsize, NULL, 0, -1, 1,
+                     NULL, NULL, NULL);
+  oarr=out->array;
+
+  /* Loop through all the rows in the given columns and check the points.*/
+  for(i=0; i<col1->size; i++)
+    {
+      /* Read the column values as a double. */
+      point[0]=selection_polygon_read_point(col1, i);
+      point[1]=selection_polygon_read_point(col2, i);
+
+      /* For '--inpolygon', if point is inside polygon, put 0, otherwise
+         1. Note that we are building a mask for the rows that must be
+         discarded, so we want '1' for the points we don't want. */
+      oarr[i] = (in1out0
+                 ? !gal_polygon_is_inside(p->polygon->array, point, psize)
+                 :  gal_polygon_is_inside(p->polygon->array, point, psize));
+
+      /* For a check
+      printf("(%f,%f): %s, %u\n", point[0], point[1], oarr[i]);
+      */
+    }
+
+  /* Return the output column. */
+  return out;
+}
+
+
+
+
+
+/* Given a string dataset and a single string, return a 'uint8_t' array
+   with the same size as the string dataset that has a '1' for all the
+   elements that are equal. */
+static gal_data_t *
+table_selection_string_eq_ne(gal_data_t *column, char *reference, int e0n1)
+{
+  gal_data_t *out;
+  uint8_t *oarr, comp;
+  size_t i, size=column->size;
+  char **strarr=column->array;
+
+  /* Allocate the output binary dataset. */
+  out=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &size, NULL, 0, -1, 1,
+                     NULL, NULL, NULL);
+  oarr=out->array;
+
+  /* Parse the values and mark the outputs IN THE OPPOSITE manner (we are
+     marking the ones that must be removed). */
+  for(i=0;i<size;++i)
+    {
+      comp=strcmp(strarr[i], reference);
+      oarr[i] = e0n1 ? (comp==0) : (comp!=0);
+    }
+
+  /* Return. */
+  return out;
+}
+
+
+
+
+
 static gal_data_t *
 table_selection_equal_or_notequal(struct tableparams *p, gal_data_t *col,
                                   int e0n1)
 {
-  double *darr;
+  void *varr;
+  char **strarr;
   size_t i, one=1;
   int numok=GAL_ARITHMETIC_NUMOK;
   int inplace=GAL_ARITHMETIC_INPLACE;
@@ -147,31 +259,50 @@ table_selection_equal_or_notequal(struct tableparams *p, gal_data_t *col,
   gal_data_t *arg = e0n1 ? p->notequal : p->equal;
 
   /* Note that this operator is used to make the "masked" array, so when
-     `e0n1==0' the operator should be `GAL_ARITHMETIC_OP_NE' and
+     'e0n1==0' the operator should be 'GAL_ARITHMETIC_OP_NE' and
      vice-versa.
 
-     For the merging with other elements, when `e0n1==0', we need the
-     `GAL_ARITHMETIC_OP_AND', but for `e0n1==1', it should be `OR'. */
+     For the merging with other elements, when 'e0n1==0', we need the
+     'GAL_ARITHMETIC_OP_AND', but for 'e0n1==1', it should be 'OR'. */
   int mergeop  = e0n1 ? GAL_ARITHMETIC_OP_OR : GAL_ARITHMETIC_OP_AND;
   int operator = e0n1 ? GAL_ARITHMETIC_OP_EQ : GAL_ARITHMETIC_OP_NE;
 
   /* First, make sure everything is OK. */
   if(arg==NULL)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us to fix the "
-          "problem at %s. `p->range' should not be NULL at this point",
+          "problem at %s. 'p->range' should not be NULL at this point",
           __func__, PACKAGE_BUGREPORT);
 
-  /* Allocate space for the value. */
-  value=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &one, NULL, 0, -1, 1,
-                     NULL, NULL, NULL);
+  /* To easily parse the given values. */
+  strarr=arg->array;
 
   /* Go through the values given to this call of the option and flag the
      elements. */
   for(i=0;i<arg->size;++i)
     {
-      darr=arg->array;
-      ((double *)(value->array))[0] = darr[i];
-      eq=gal_arithmetic(operator, 1, numok, col, value);
+      /* Write the value  */
+      if(col->type==GAL_TYPE_STRING)
+        eq=table_selection_string_eq_ne(col, strarr[i], e0n1);
+      else
+        {
+          /* Allocate the value dataset. */
+          value=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &one, NULL, 0, -1, 1,
+                               NULL, NULL, NULL);
+          varr=value->array;
+
+          /* Read the stored string as a float64. */
+          if( gal_type_from_string(&varr, strarr[i], GAL_TYPE_FLOAT64) )
+            {
+              fprintf(stderr, "%s couldn't be read as a number.\n", strarr[i]);
+              exit(EXIT_FAILURE);
+            }
+
+          /* Mark the rows that are equal (irrespective of the column's
+             original numerical datatype). */
+          eq=gal_arithmetic(operator, 1, numok, col, value);
+        }
+
+      /* Merge the results with (possible) previous results. */
       if(out)
         {
           out=gal_arithmetic(mergeop, 1, inplace, out, eq);
@@ -189,8 +320,10 @@ table_selection_equal_or_notequal(struct tableparams *p, gal_data_t *col,
   }
   */
 
+
   /* Move the main pointer to the next possible call of the given
-     option. With this, we can safely free `arg' at this point. */
+     option. Note that 'arg' already points to 'p->equal' or 'p->notequal',
+     so it will automatically be freed with the next step.*/
   if(e0n1) p->notequal=p->notequal->next;
   else     p->equal=p->equal->next;
 
@@ -229,6 +362,14 @@ table_selection(struct tableparams *p)
           addmask=table_selection_range(p, tmp->col);
           break;
 
+        /* '--inpolygon' and '--outpolygon' need two columns. */
+        case SELECT_TYPE_INPOLYGON:
+        case SELECT_TYPE_OUTPOLYGON:
+          addmask=table_selection_polygon(p, tmp->col, tmp->next->col,
+                                          tmp->type==SELECT_TYPE_INPOLYGON);
+          tmp=tmp->next;
+          break;
+
         case SELECT_TYPE_EQUAL:
           addmask=table_selection_equal_or_notequal(p, tmp->col, 0);
           break;
@@ -348,10 +489,10 @@ table_sort(struct tableparams *p)
           "TIP: if you know the columns contents are all numbers that are "
           "just stored as strings, you can use this program to save the "
           "table as a text file, modify the column meta-data (for example "
-          "to type `i32' or `f32' instead of `strN'), then use this "
+          "to type 'i32' or 'f32' instead of 'strN'), then use this "
           "program again to save it as a FITS table.\n\n"
           "For more on column metadata in plain text format, please run "
-          "the following command (or see the `Gnuastro text table format "
+          "the following command (or see the 'Gnuastro text table format "
           "section of the book/manual):\n\n"
           "    $ info gnuastro \"gnuastro text table format\"");
 
@@ -371,7 +512,7 @@ table_sort(struct tableparams *p)
       case GAL_TYPE_FLOAT64: qsortfn=gal_qsort_index_single_float64_d; break;
       default:
         error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-              "the problem. The code `%u' wasn't recognized as a data type",
+              "the problem. The code '%u' wasn't recognized as a data type",
               __func__, PACKAGE_BUGREPORT, p->sortcol->type);
       }
   else
@@ -389,7 +530,7 @@ table_sort(struct tableparams *p)
       case GAL_TYPE_FLOAT64: qsortfn=gal_qsort_index_single_float64_i; break;
       default:
         error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-              "the problem. The code `%u' wasn't recognized as a data type",
+              "the problem. The code '%u' wasn't recognized as a data type",
               __func__, PACKAGE_BUGREPORT, p->sortcol->type);
       }
 
@@ -397,7 +538,7 @@ table_sort(struct tableparams *p)
   gal_qsort_index_single=p->sortcol->array;
   qsort(perm->array, perm->size, sizeof *s, qsortfn);
 
-  /* For a check (only on float32 type `sortcol'):
+  /* For a check (only on float32 type 'sortcol'):
   {
     float *f=p->sortcol->array;
     sf=(s=perm->array)+perm->size;
@@ -430,9 +571,9 @@ table_head_tail(struct tableparams *p)
     {
       /* If we are dealing with strings, we'll need to free the strings
          that the columns that will not be used point to (outside the
-         allocated array directly `gal_data_t'). We don't have to worry
+         allocated array directly 'gal_data_t'). We don't have to worry
          about the space for the actual pointers (they will be free'd by
-         `free' in any case, since they are in the initially allocated
+         'free' in any case, since they are in the initially allocated
          array).*/
       if(col->type==GAL_TYPE_STRING)
         {
@@ -445,8 +586,8 @@ table_head_tail(struct tableparams *p)
           for(i=start; i<end; ++i) { free(strarr[i]); strarr[i]=NULL; }
         }
 
-      /* For `--tail', we'll need to bring the last columns to the
-         start. Note that we are using `memmove' because we want to be safe
+      /* For '--tail', we'll need to bring the last columns to the
+         start. Note that we are using 'memmove' because we want to be safe
          with overlap. */
       if(p->tail!=GAL_BLANK_SIZE_T)
         memmove(col->array,
@@ -466,6 +607,160 @@ table_head_tail(struct tableparams *p)
 
 
 
+/*This function concatenates two table column wise .
+ It  attaches catcolumn table at the back of first table */
+static void
+table_catcolumn(struct tableparams *p)
+{
+  size_t counter=1;
+  gal_list_str_t *filell, *hdull;
+  gal_data_t *tocat, *final, *newcol;
+  char *tmpname, *hdu=NULL, cstr[100];
+  struct gal_options_common_params *cp=&p->cp;
+
+  /* Go over all the given files. */
+  hdull=p->catcolumnhdu;
+  for(filell=p->catcolumnfile; filell!=NULL; filell=filell->next)
+    {
+      /* Set the HDU (not necessary for non-FITS tables). */
+      if(gal_fits_name_is_fits(filell->v))
+        {
+          if(hdull) { hdu=hdull->v; hdull=hdull->next; }
+          else
+            error(EXIT_FAILURE, 0, "not enough '--catcolumnhdu's (or '-u'). "
+                  "For every FITS table given to '--catcolumnfile'. A call to "
+                  "'--catcolumnhdu' is necessary to identify its "
+                  "HDU/extension");
+        }
+      else hdu=NULL;
+
+      /* Read the catcolumn table. */
+      tocat=gal_table_read(filell->v, hdu, NULL, p->catcolumns, cp->searchin,
+                           cp->ignorecase, cp->minmapsize, p->cp.quietmmap,
+                           NULL);
+
+      /* Check the number of rows. */
+      if(tocat->dsize[0]!=p->table->dsize[0])
+        error(EXIT_FAILURE, 0, "%s: incorrect number of rows. The table given "
+              "to '--catcolumn' must have the same number of rows as the "
+              "main argument (after all row-selections have been applied), "
+              "but they have %zu and %zu rows respectively",
+              gal_fits_name_save_as_string(filell->v, hdu), tocat->dsize[0],
+              p->table->dsize[0]);
+
+      /* Append a counter to the column names because this option is most
+         often used with columns that have a similar name and it would help
+         the user if the output doesn't have multiple columns with same
+         name. */
+      if(p->catcolumnrawname==0)
+        for(newcol=tocat; newcol!=NULL; newcol=newcol->next)
+          if(newcol->name)
+            {
+              /* Add the counter suffix to the column name. */
+              sprintf(cstr, "-%zu", counter);
+              tmpname=gal_checkset_malloc_cat(newcol->name, cstr);
+
+              /* Free the old name and put in the new one. */
+              free(newcol->name);
+              newcol->name=tmpname;
+            }
+
+      /* Find the final column of the main table and add this table.*/
+      final=gal_list_data_last(p->table);
+      final->next=tocat;
+      ++counter;
+    }
+}
+
+
+
+
+
+void
+table_colmetadata(struct tableparams *p)
+{
+  char **strarr;
+  gal_data_t *meta, *col;
+  size_t counter, *colnum;
+
+  /* Loop through all the given updates and implement them. */
+  for(meta=p->colmetadata;meta!=NULL;meta=meta->next)
+    {
+      /* If the given column specifier is a name (not parse-able as a
+         number), then this condition will fail. */
+      colnum=NULL;
+      if( gal_type_from_string((void **)(&colnum), meta->name,
+                               GAL_TYPE_SIZE_T) )
+        {
+          /* We have been given a string, so find the first column that has
+             the same name. */
+          for(col=p->table; col!=NULL; col=col->next)
+            if(!strcmp(col->name, meta->name)) break;
+        }
+      /* The column specifier is a number. */
+      else
+        {
+          /* Go over the columns and find the one with this counter. */
+          counter=1;
+          for(col=p->table; col!=NULL; col=col->next)
+            if(counter++==colnum[0]) break;
+
+          /* Clean up the space that was allocated for 'colnum' (its not
+             allocated when the given value was a string). */
+          free(colnum);
+        }
+
+      /* If a match was found, then 'col' should not be NULL. */
+      if(col==NULL)
+        error(EXIT_FAILURE, 0, "no column found for '%s' (given to "
+              "'--colmetadata'). Columns can either be specified by "
+              "their position in the output table (integer counter, "
+              "starting from 1), or their name (the first column "
+              "found with the given name will be used)", meta->name);
+
+      /* The matching column is found and we know that atleast one value is
+         already given (otherwise 'gal_options_parse_name_and_values' would
+         abort the program). The first given string is the new name. */
+      strarr=meta->array;
+      if(col->name) free(col->name);
+      gal_checkset_allocate_copy(strarr[0], &col->name);
+
+      /* If more than one string is given, the second one is the new
+         unit. */
+      if(meta->size>1)
+        {
+          /* Replace the unit. */
+          if(col->unit) free(col->unit);
+          gal_checkset_allocate_copy(strarr[1], &col->unit);
+
+          /* The next element is the comment of the column. */
+          if(meta->size>2)
+            {
+              if(col->comment) free(col->comment);
+              gal_checkset_allocate_copy(strarr[2], &col->comment);
+            }
+        }
+    }
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 /**************************************************************/
 /***************       Top function         *******************/
 /**************************************************************/
@@ -486,6 +781,12 @@ table(struct tableparams *p)
   if(p->outcols)
     arithmetic_operate(p);
 
+  /* Concatenate the columns of tables (if required)*/
+  if(p->catcolumnfile) table_catcolumn(p);
+
+  /* If column metadata should be updated, do it just before writing. */
+  if(p->colmetadata) table_colmetadata(p);
+
   /* Write the output. */
   gal_table_write(p->table, NULL, p->cp.tableformat, p->cp.output,
                   "TABLE", p->colinfoinstdout);
