@@ -34,12 +34,14 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Configuration macros: */
 
-#define GAL_CONFIG_VERSION                  "0.12"
+#define GAL_CONFIG_VERSION                  "0.12.37-5018"
 
 #define GAL_CONFIG_HAVE_LIBGIT2             yes
 
 #define GAL_CONFIG_HAVE_FITS_IS_REENTRANT   @HAVE_FITS_IS_REENTRANT@
 #define GAL_CONFIG_HAVE_WCSLIB_VERSION      1
+#define GAL_CONFIG_HAVE_WCSLIB_DIS_H        1
+#define GAL_CONFIG_HAVE_WCSLIB_MJDREF       1
 
 #define GAL_CONFIG_HAVE_PTHREAD_BARRIER     1
 
