@@ -521,10 +521,10 @@ gal_options_read_interpmetric(struct argp_option *option, char *arg,
     {
       switch(*(uint8_t *)(option->value))
         {
-        case GAL_INTERPOLATE_CLOSE_METRIC_RADIAL:
+        case GAL_INTERPOLATE_NEIGHBORS_METRIC_RADIAL:
           gal_checkset_allocate_copy("radial", &str);
           break;
-        case GAL_INTERPOLATE_CLOSE_METRIC_MANHATTAN:
+        case GAL_INTERPOLATE_NEIGHBORS_METRIC_MANHATTAN:
           gal_checkset_allocate_copy("manhattan", &str);
           break;
         default:
@@ -542,9 +542,9 @@ gal_options_read_interpmetric(struct argp_option *option, char *arg,
 
       /* Set the value. */
       if(       !strcmp(arg, "radial") )
-        *(uint8_t *)(option->value) = GAL_INTERPOLATE_CLOSE_METRIC_RADIAL;
+        *(uint8_t *)(option->value) = GAL_INTERPOLATE_NEIGHBORS_METRIC_RADIAL;
       else if ( !strcmp(arg, "manhattan") )
-        *(uint8_t *)(option->value) = GAL_INTERPOLATE_CLOSE_METRIC_MANHATTAN;
+        *(uint8_t *)(option->value) = GAL_INTERPOLATE_NEIGHBORS_METRIC_MANHATTAN;
       else
         error_at_line(EXIT_FAILURE, 0, filename, lineno, "'%s' (value to "
                       "'%s' option) isn't valid. Currently only 'radial' "
@@ -1133,11 +1133,10 @@ gal_options_read_sigma_clip(struct argp_option *option, char *arg,
 
 
 
-/* Parse name and (float64) values:  name,value1,value2,value3,...
+/* Parse name and (string/float64) values:  name,value1,value2,value3,...
 
    The output is a 'gal_data_t', where the 'name' is the given name and the
-   values are in its array (of 'float64' type).
- */
+   values are in its array (of 'char *' or 'float64' type). */
 static void *
 gal_options_parse_name_and_values(struct argp_option *option, char *arg,
                                   char *filename, size_t lineno, void *junk,
@@ -1223,10 +1222,10 @@ gal_options_parse_name_and_values(struct argp_option *option, char *arg,
           */
         }
       else
-        error(EXIT_FAILURE, 0, "'--%s' requires a string of numbers "
+        error(EXIT_FAILURE, 0, "'--%s' requires a series of %s "
               "(separated by ',' or ':') following its first argument, "
               "please run with '--help' for more information",
-              option->name);
+              option->name, str0_f641?"numbers":"strings");
 
       /* Our job is done, return NULL. */
       return NULL;
