@@ -32,6 +32,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <assert.h>
 
 #include <gsl/gsl_linalg.h>
+#include <wcslib/wcsmath.h>
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/tile.h>
@@ -40,8 +41,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/dimension.h>
 #include <gnuastro/permutation.h>
 
-
-
+#if GAL_CONFIG_HAVE_WCSLIB_DIS_H
+#include <wcslib/dis.h>
+#include <gnuastro-internal/wcsdistortion.h>
+#endif
 
 
 
@@ -267,6 +270,345 @@ gal_wcs_read(char *filename, char *hdu, size_t hstartwcs,
 
 
 
+
+/*************************************************************
+ ***********               Write WCS               ***********
+ *************************************************************/
+void
+gal_wcs_write(struct wcsprm *wcs, char *filename,
+              char *extname, gal_fits_list_key_t *headers,
+              char *program_string)
+{
+  char *wcsstr;
+  size_t ndim=0;
+  fitsfile *fptr;
+  long *naxes=NULL;
+  int status=0, nkeyrec;
+
+  /* Small sanity checks */
+  if(wcs==NULL)
+    error(EXIT_FAILURE, 0, "%s: input WCS is NULL", __func__);
+  if( gal_fits_name_is_fits(filename)==0 )
+    error(EXIT_FAILURE, 0, "%s: not a FITS suffix", filename);
+
+  /* Open the file for writing */
+  fptr=gal_fits_open_to_write(filename);
+
+  /* Create the FITS file. */
+  fits_create_img(fptr, gal_fits_type_to_bitpix(GAL_TYPE_UINT8),
+                  ndim, naxes, &status);
+  gal_fits_io_error(status, NULL);
+
+  /* Remove the two comment lines put by CFITSIO. Note that in some cases,
+     it might not exist. When this happens, the status value will be
+     non-zero. We don't care about this error, so to be safe, we will just
+     reset the status variable after these calls. */
+  fits_delete_key(fptr, "COMMENT", &status);
+  fits_delete_key(fptr, "COMMENT", &status);
+  status=0;
+
+  if(extname)
+    fits_write_key(fptr, TSTRING, "EXTNAME", extname, "", &status);
+
+  /* Decompose the 'PCi_j' matrix and 'CDELTi' vector. */
+  gal_wcs_decompose_pc_cdelt(wcs);
+
+  /* Clean up small errors in the PC matrix and CDELT values. */
+  gal_wcs_clean_errors(wcs);
+
+  /* Convert the WCS information to text. */
+  status=wcshdo(WCSHDO_safe, wcs, &nkeyrec, &wcsstr);
+  if(status)
+    error(0, 0, "%s: WARNING: WCSLIB error, no WCS in output.\n"
+          "wcshdu ERROR %d: %s", __func__, status,
+          wcs_errmsg[status]);
+  else
+    gal_fits_key_write_wcsstr(fptr, wcsstr, nkeyrec);
+  status=0;
+
+  /* Write all the headers and the version information. */
+  gal_fits_key_write_version_in_ptr(&headers, program_string, fptr);
+
+  /* Close the FITS file. */
+  fits_close_file(fptr, &status);
+  gal_fits_io_error(status, NULL);
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/*************************************************************
+ ***********              Distortions              ***********
+ *************************************************************/
+int
+gal_wcs_distortion_from_string(char *distortion)
+{
+  if(      !strcmp(distortion,"TPD") ) return GAL_WCS_DISTORTION_TPD;
+  else if( !strcmp(distortion,"SIP") ) return GAL_WCS_DISTORTION_SIP;
+  else if( !strcmp(distortion,"TPV") ) return GAL_WCS_DISTORTION_TPV;
+  else if( !strcmp(distortion,"DSS") ) return GAL_WCS_DISTORTION_DSS;
+  else if( !strcmp(distortion,"WAT") ) return GAL_WCS_DISTORTION_WAT;
+  else
+    error(EXIT_FAILURE, 0, "WCS distortion name '%s' not recognized, "
+          "currently recognized names are 'TPD', 'SIP', 'TPV', 'DSS' and "
+          "'WAT'", distortion);
+
+  /* Control should not reach here. */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
+        "problem. Control should not reach the end of this function",
+        __func__, PACKAGE_BUGREPORT);
+  return GAL_WCS_DISTORTION_INVALID;
+}
+
+
+
+
+
+char *
+gal_wcs_distortion_to_string(int distortion)
+{
+  /* Return the proper literal string. */
+  switch(distortion)
+    {
+    case GAL_WCS_DISTORTION_TPD: return "TPD";
+    case GAL_WCS_DISTORTION_SIP: return "SIP";
+    case GAL_WCS_DISTORTION_TPV: return "TPV";
+    case GAL_WCS_DISTORTION_DSS: return "DSS";
+    case GAL_WCS_DISTORTION_WAT: return "WAT";
+    default:
+      error(EXIT_FAILURE, 0, "WCS distortion id '%d' isn't recognized",
+            distortion);
+    }
+
+  /* Control should not reach here. */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
+        "problem. Control should not reach the end of this function",
+        __func__, PACKAGE_BUGREPORT);
+  return NULL;
+}
+
+
+
+
+
+/* Check the type of distortion present and return the appropriate
+   integer based on `enum gal_wcs_distortion`.
+
+   Parameters:
+    struct wcsprm *wcs - The wcs parameters of the fits file.
+
+   Return:
+    int out_distortion - The type of distortion present. */
+int
+gal_wcs_distortion_identify(struct wcsprm *wcs)
+{
+#if GAL_CONFIG_HAVE_WCSLIB_DIS_H
+  struct disprm *dispre=NULL;
+  struct disprm *disseq=NULL;
+
+  /* Small sanity check. */
+  if(wcs==NULL) return GAL_WCS_DISTORTION_INVALID;
+
+  /* To help in reading. */
+  disseq=wcs->lin.disseq;
+  dispre=wcs->lin.dispre;
+
+  /* Check if distortion present. */
+  if( disseq==NULL && dispre==NULL ) return GAL_WCS_DISTORTION_INVALID;
+
+  /* Check the type of distortion.
+
+     As mentioned in the WCS paper IV section 2.4.2 available at
+     https://www.atnf.csiro.au/people/mcalabre/WCS/dcs_20040422.pdf, the
+     DPja and DQia keywords are used to record the parameters required by
+     the prior and sequent distortion functions respectively.
+
+     Now, as mentioned in dis.h file reference section in WCSLIB manual
+     given here
+     https://www.atnf.csiro.au/people/mcalabre/WCS/wcslib/dis_8h.html, TPV,
+     DSS, and WAT are sequent polynomial distortions, while SIP is prior
+     polynomial distortion. TPD is a superset of all these distortions and
+     hence can be used both as a prior and sequent distortion polynomial.
+
+     References and citations:
+     "Representations of distortions in FITS world coordinate systems",
+     Calabretta, M.R. et al. (WCS Paper IV, draft dated 2004/04/22)
+      */
+
+  if( dispre != NULL )
+    {
+      if(      !strcmp(*dispre->dtype, "SIP") ) return GAL_WCS_DISTORTION_SIP;
+      else if( !strcmp(*dispre->dtype, "TPD") ) return GAL_WCS_DISTORTION_TPD;
+      else
+        error(EXIT_FAILURE, 0, "%s: distortion '%s' isn't recognized in "
+              "the 'dispre' structure of the given 'wcsprm'", __func__,
+              *dispre->dtype);
+    }
+  else if( disseq != NULL )
+    {
+      if(      !strcmp(*disseq->dtype, "TPV") ) return GAL_WCS_DISTORTION_TPV;
+      else if( !strcmp(*disseq->dtype, "TPD") ) return GAL_WCS_DISTORTION_TPD;
+      else if( !strcmp(*disseq->dtype, "DSS") ) return GAL_WCS_DISTORTION_DSS;
+      else if( !strcmp(*disseq->dtype, "WAT") ) return GAL_WCS_DISTORTION_WAT;
+      else
+        error(EXIT_FAILURE, 0, "%s: distortion '%s' isn't recognized in "
+              "the 'disseq' structure of the given 'wcsprm'", __func__,
+              *dispre->dtype);
+    }
+
+  /* Control should not reach here. */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to fix "
+        "the problem. Control should not reach the end of this function",
+        __func__, PACKAGE_BUGREPORT);
+#else
+  /* The 'wcslib/dis.h' isn't present. */
+  error(EXIT_FAILURE, 0, "%s: the installed version of WCSLIB on this "
+        "system doesn't have the 'dis.h' header! Thus Gnuastro can't do "
+        "distortion-related operations on the world coordinate system "
+        "(WCS). To use these features, please upgrade your version "
+        "of WCSLIB and re-build Gnuastro", __func__);
+#endif
+  return GAL_WCS_DISTORTION_INVALID;
+}
+
+
+
+
+
+
+
+
+
+
+/* Convert a given distrotion type to other.
+
+  Parameters:
+    struct wcsprm *wcs - The wcs parameters of the fits file.
+    int out_distortion - The desired output distortion.
+    size_t* fitsize    - The size of the array along each dimension.
+
+  Return:
+    struct wcsprm *outwcs - The transformed wcs parameters in the
+                            required distortion type. */
+struct wcsprm *
+gal_wcs_distortion_convert(struct wcsprm *inwcs, int outdisptype,
+                           size_t *fitsize)
+{
+#if GAL_CONFIG_HAVE_WCSLIB_DIS_H
+  struct wcsprm *outwcs=NULL;
+  int indisptype=gal_wcs_distortion_identify(inwcs);
+
+  /* Make sure we have a PC+CDELT structure in the input WCS. */
+  gal_wcs_decompose_pc_cdelt(inwcs);
+
+  /* If the input and output types are the same, just copy the input,
+     otherwise, do the conversion. */
+  if(indisptype==outdisptype) outwcs=gal_wcs_copy(inwcs);
+  else
+    switch(indisptype)
+      {
+      /* If there is no distortion in the input, just return a
+         newly-allocated copy. */
+      case GAL_WCS_DISTORTION_INVALID: outwcs=gal_wcs_copy(inwcs); break;
+
+      /* Input's distortion is SIP. */
+      case GAL_WCS_DISTORTION_SIP:
+        switch(outdisptype)
+          {
+          case GAL_WCS_DISTORTION_TPV:
+            outwcs=gal_wcsdistortion_sip_to_tpv(inwcs);
+            break;
+          default:
+            error(EXIT_FAILURE, 0, "%s: conversion from %s to %s is not yet "
+                  "supported. Please contact us at '%s'", __func__,
+                  gal_wcs_distortion_to_string(indisptype),
+                  gal_wcs_distortion_to_string(outdisptype),
+                  PACKAGE_BUGREPORT);
+              }
+        break;
+
+      /* Input's distortion is TPV. */
+      case GAL_WCS_DISTORTION_TPV:
+        switch(outdisptype)
+          {
+          case GAL_WCS_DISTORTION_SIP:
+            if(fitsize==NULL)
+              error(EXIT_FAILURE, 0, "%s: the size array is necessary "
+                    "for this conversion", __func__);
+            outwcs=gal_wcsdistortion_tpv_to_sip(inwcs, fitsize);
+            break;
+          default:
+            error(EXIT_FAILURE, 0, "%s: conversion from %s to %s is not yet "
+                  "supported. Please contact us at '%s'", __func__,
+                  gal_wcs_distortion_to_string(indisptype),
+                  gal_wcs_distortion_to_string(outdisptype),
+                  PACKAGE_BUGREPORT);
+          }
+        break;
+
+      /* Input's distortion is not yet supported.. */
+      case GAL_WCS_DISTORTION_TPD:
+      case GAL_WCS_DISTORTION_DSS:
+      case GAL_WCS_DISTORTION_WAT:
+        error(EXIT_FAILURE, 0, "%s: input %s distortion is not yet "
+              "supported. Please contact us at '%s'", __func__,
+              gal_wcs_distortion_to_string(indisptype),
+              PACKAGE_BUGREPORT);
+
+      /* A bug! This distortion is not yet recognized. */
+      default:
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+              "the problem. The identifier '%d' is not recognized as a "
+              "distortion", __func__, PACKAGE_BUGREPORT, indisptype);
+      }
+
+  /* Return the converted WCS. */
+  return outwcs;
+#else
+  /* The 'wcslib/dis.h' isn't present. */
+  error(EXIT_FAILURE, 0, "%s: the installed version of WCSLIB on this "
+        "system doesn't have the 'dis.h' header! Thus Gnuastro can't do "
+        "distortion-related operations on the world coordinate system "
+        "(WCS). To use these features, please upgrade your version "
+        "of WCSLIB and re-build Gnuastro", __func__);
+  return NULL;
+#endif
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 /**************************************************************/
 /**********              Utilities                 ************/
@@ -594,6 +936,46 @@ gal_wcs_warp_matrix(struct wcsprm *wcs)
 
 
 
+/* Clean up small/negligible errros that are clearly caused by measurement
+   errors in the PC and CDELT elements. */
+void
+gal_wcs_clean_errors(struct wcsprm *wcs)
+{
+  double crdcheck=NAN;
+  size_t i, crdnum=0, ndim=wcs->naxis;
+  double mean, crdsum=0, sum=0, min=FLT_MAX, max=0;
+  double *pc=wcs->pc, *cdelt=wcs->cdelt, *crder=wcs->crder;
+
+  /* First clean up CDELT: if the CRDER keyword is set, then we'll use that
+     as a reference, if not, we'll use the absolute floating point error
+     defined in 'GAL_WCS_FLTERR'. */
+  for(i=0; i<ndim; ++i)
+    {
+      sum+=cdelt[i];
+      if(cdelt[i]>max) max=cdelt[i];
+      if(cdelt[i]<min) min=cdelt[i];
+      if(crder[i]!=UNDEFINED) {++crdnum; crdsum=crder[i];}
+    }
+  mean=sum/ndim;
+  crdcheck = crdnum ? crdsum/crdnum : GAL_WCS_FLTERROR;
+  if( (max-min)/mean < crdcheck )
+    for(i=0; i<ndim; ++i)
+      cdelt[i]=mean;
+
+  /* Now clean up the PC elements. If the diagonal elements are too close
+     to 0, 1, or -1, set them to 0 or 1 or -1. */
+  if(pc)
+    for(i=0;i<ndim*ndim;++i)
+      {
+        if(      fabs(pc[i] -  0 ) < GAL_WCS_FLTERROR ) pc[i]=0;
+        else if( fabs(pc[i] -  1 ) < GAL_WCS_FLTERROR ) pc[i]=1;
+        else if( fabs(pc[i] - -1 ) < GAL_WCS_FLTERROR ) pc[i]=-1;
+      }
+}
+
+
+
+
 
 /* According to the FITS standard, in the 'PCi_j' WCS formalism, the matrix
    elements m_{ij} are encoded in the 'PCi_j' keywords and the scale
@@ -747,7 +1129,10 @@ gal_wcs_pixel_scale(struct wcsprm *wcs)
           }
 
       /* Do the check, print warning and make correction. */
-      if(maxrow!=minrow && maxrow/minrow>1e5 && warning_printed==0)
+      if(maxrow!=minrow
+         && maxrow/minrow>1e5    /* The difference between elements is large */
+         && maxrow/minrow<GAL_WCS_FLTERROR
+         && warning_printed==0)
         {
           fprintf(stderr, "\nWARNING: The input WCS matrix (possibly taken "
                   "from the FITS header keywords starting with 'CD' or 'PC') "
